package cn.timer.api.controller.disk;

import cn.timer.api.bean.disk.DiskCatalogue;
import cn.timer.api.bean.disk.DiskCatalogueFiles;
import cn.timer.api.bean.disk.DiskCloudDiskUser;
import cn.timer.api.bean.disk.DiskFiles;
import cn.timer.api.bean.kqgl.UserAttendanceRel;
import cn.timer.api.bean.qyzx.QyzxEmpLogin;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.config.enuminterface.JxglEnumInterface;
import cn.timer.api.config.exception.CustomException;
import cn.timer.api.controller.disk.sevice.DiskCatalogueService;
import cn.timer.api.controller.disk.sevice.DiskCloudDiskUserService;
import cn.timer.api.controller.disk.sevice.DiskFilesService;
import cn.timer.api.dto.disk.DiskCatalogueDto;
import cn.timer.api.dto.disk.DiskCatalogueParam;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.transaction.Transactional;
import java.util.*;


/**
 * 云盘-资源目录表
 *
 * @author wuqingjun
 * @email 284718418@qq.com
 * @date 2021-12-27 10:05:49
 */
@Api(tags = "云盘")
@Transactional(rollbackOn = Exception.class)
@RestController
@RequestMapping(value = "/disk/catalogue", produces = {"application/json"})
public class DiskCatalogueController {

    @Autowired
    private DiskCatalogueService diskCatalogueService;

    @Autowired
    private DiskFilesService diskFilesService;

    @Autowired
    private DiskCloudDiskUserService diskCloudDiskUserService;

    /**
     * 云盘-新建文件夹
     *
     * @param userBean
     * @return
     */
    @PostMapping(value = "/add_catalogue")
    @ApiOperation(value = "1.新建文件夹或共享空间", httpMethod = "POST", notes = "新建文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 1)
    public Result<Object> addcatalogue(@CurrentUser UserBean userBean, @Validated @RequestBody DiskCatalogueDto diskCatalogueDto) {
        try{
            DiskCatalogue diskCatalogue = new DiskCatalogue();
            DiskCatalogue catalogue = DiskCatalogue.builder().id(diskCatalogueDto.getParentId()).build().selectById();

            // 云盘修改 权限判断
            if(JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueDto.getType()) &&
                    !StringUtils.isEmpty(diskCatalogueDto.getId()) && diskCatalogueDto.getId() > 0){
                diskCatalogueDto.setUserId(userBean.getEmpNum());
                if(!diskCloudDiskUserService.getDiskUserRole(diskCatalogueDto,DiskCatalogue.builder().id(diskCatalogueDto.getId()).build().selectById())){
                    return ResultUtil.error("无权限，操作失败!");
                }
            }



            BeanUtils.copyProperties(diskCatalogueDto, diskCatalogue);
            String parentIds = "";
            if (!StringUtils.isEmpty(catalogue)) {
                parentIds = catalogue.getParentIds();
                diskCatalogue.setParentIds(catalogue.getParentId() > 0 ? parentIds + "," + catalogue.getId() : catalogue.getId().toString());
            }else {
                diskCatalogue.setParentIds(parentIds);
            }
            diskCatalogue.setCreateUserId(userBean.getEmpNum());
            diskCatalogue.setOrgId(userBean.getOrgCode());
            diskCatalogue.setUserName(userBean.getQyzxEmpLogin().getUsername());
            diskCatalogue.setUpdateUserId(userBean.getEmpNum());
            Boolean count = diskCatalogue.insertOrUpdate();
            // 云盘修改
            if(!StringUtils.isEmpty(diskCatalogueDto.getId()) && diskCatalogueDto.getId() > 0
                    && JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueDto.getType())){

                // 删除之前的成员 全部重新添加
                DiskCloudDiskUser.builder().build().delete(new QueryWrapper<DiskCloudDiskUser>()
                        .lambda().eq(DiskCloudDiskUser::getOrgId, userBean.getOrgCode())
                        .eq(DiskCloudDiskUser::getCatalogueId, diskCatalogue.getId()));
                if(!CollectionUtils.isEmpty(diskCatalogueDto.getDiskCloudDiskUsers())){
                    for (DiskCloudDiskUser entity : diskCatalogueDto.getDiskCloudDiskUsers()) {
                        QyzxEmpLogin login = QyzxEmpLogin.builder().build().selectOne(new QueryWrapper<QyzxEmpLogin>().select("username","head_url").eq("id", entity.getUserId()));
                        entity.setCatalogueId(diskCatalogue.getId());
                        entity.setUserPower(entity.getUserPower());
                        entity.setOrgId(userBean.getOrgCode());
                        entity.setUserName(login.getUsername());
                        entity.setUserHeadUrl(StringUtils.isEmpty(login.getHeadUrl())?"":login.getHeadUrl());
                        entity.insert();
                    }
                }

            }
            // 云盘新增
            if((StringUtils.isEmpty(diskCatalogueDto.getId()) || diskCatalogueDto.getId() <= 0)
                    && JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueDto.getType())
                    && !CollectionUtils.isEmpty(diskCatalogueDto.getDiskCloudDiskUsers())){
                for (DiskCloudDiskUser entity : diskCatalogueDto.getDiskCloudDiskUsers()) {
                    QyzxEmpLogin login = QyzxEmpLogin.builder().build().selectOne(new QueryWrapper<QyzxEmpLogin>().select("username","head_url").eq("id", entity.getUserId()));
                    entity.setCatalogueId(diskCatalogue.getId());
                    entity.setUserPower(entity.getUserPower());
                    entity.setOrgId(userBean.getOrgCode());
                    entity.setUserName(login.getUsername());
                    entity.setUserHeadUrl(StringUtils.isEmpty(login.getHeadUrl())?"":login.getHeadUrl());
                    entity.insert();
                }

            }
            if (count) {
                return ResultUtil.data(diskCatalogue, "新建文件夹成功!");
            } else {
                return ResultUtil.error("新建文件夹失败!");
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("新建文件夹或共享空间异常");
        }
    }

    /**
     * 云盘-我的文件文件夹或共享空间列表
     *
     * @param userBean
     * @param type
     * @return
     */
    @GetMapping(value = "/myfile_list")
    @ApiOperation(value = "2.我的文件文件夹或共享空间列表", httpMethod = "GET", notes = "文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 2)
    public Result<Object> myfilelist(@CurrentUser UserBean userBean,
                                     @ApiParam("文件文件夹或共享空间ID") @RequestParam(required = true, defaultValue = "0") Integer type) {
        List<DiskCatalogue> list = diskCatalogueService.getDiskCatalogueList(userBean.getOrgCode(), userBean.getEmpNum(), type);
        return ResultUtil.data(list);
    }

    /**
     * 云盘-我的文件列表
     *
     * @param userBean
     * @param diskCatalogueParam
     * @return
     */
    @GetMapping(value = "/myfile")
    @ApiOperation(value = "我的文件夹列表", httpMethod = "GET", notes = "文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 2)
    public Result<Object> myfile(@CurrentUser UserBean userBean, DiskCatalogueParam diskCatalogueParam) {
        try {
            diskCatalogueParam.setUserId(userBean.getEmpNum());
            diskCatalogueParam.setOrgId(userBean.getOrgCode());
            // 获取文件夹
            List<DiskCatalogue> diskCatalogueList = DiskCatalogue.builder().build().selectList(new QueryWrapper<DiskCatalogue>()
                    .lambda().eq(DiskCatalogue::getOrgId, userBean.getOrgCode())
                    .eq(DiskCatalogue::getCreateUserId, userBean.getEmpNum())
                    .eq(DiskCatalogue::getParentId, diskCatalogueParam.getParentId())
                    .eq(DiskCatalogue::getType, diskCatalogueParam.getType())
                    .eq(DiskCatalogue::getDeleteFlag, 0)
            );
            if(StringUtils.isEmpty(diskCatalogueParam.getParentId()) || diskCatalogueParam.getParentId() == 0
            && JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueParam.getType())){
                //空间 获取有权限的
                List<DiskCatalogue> diskCatalogueList2 = diskCloudDiskUserService.getDiskCatalogueList(diskCatalogueParam);
                if(!CollectionUtils.isEmpty(diskCatalogueList2)){
                    diskCatalogueList.addAll(diskCatalogueList2);
                }
            }
            // 获取文件夹下所有文件
            List<DiskFiles> filesList = diskFilesService.getDiskFiles(diskCatalogueParam);
            // 获取面包屑
            List<DiskCatalogue> breadcrumbDataList = null;

            DiskCatalogue catalogue = DiskCatalogue.builder().id(diskCatalogueParam.getParentId()).build().selectById();

            if(!StringUtils.isEmpty(catalogue)){
                List<Integer> ids = new ArrayList<>();
                if(!StringUtils.isEmpty(catalogue.getParentIds())){
                    String[] list = catalogue.getParentIds().split(",");
                    for (String str :list) {
                        ids.add(Integer.parseInt(str));
                    }
                }
                ids.add(catalogue.getId());
                breadcrumbDataList = DiskCatalogue.builder().build().selectList(new QueryWrapper<DiskCatalogue>()
                        .lambda().in(DiskCatalogue::getId,ids)
                        .eq(DiskCatalogue::getOrgId, userBean.getOrgCode())
                        //.eq(DiskCatalogue::getCreateUserId, userBean.getEmpNum())
                        .eq(DiskCatalogue::getDeleteFlag, 0)
                );
            }
            Map<String,Object> map = new HashMap<>(3);
            map.put("diskCatalogueData",diskCatalogueList);
            map.put("diskFileData",filesList);
            map.put("breadcrumbData",breadcrumbDataList);
            return ResultUtil.data(map);
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("获取我的文件夹列表异常");
        }
    }

    /**
     * 	获取当前云空间成员
     */
    @GetMapping(value="/user/{id}")
    @ApiOperation(value = "获取当前云空间成员", httpMethod = "GET", notes = "获取当前云空间成员")
    public Result<Object> getAttUserGroupList(@CurrentUser UserBean userBean, @PathVariable("id") Integer id) {
        List<DiskCloudDiskUser> userattgor = DiskCloudDiskUser.builder().build().selectList(new QueryWrapper<DiskCloudDiskUser>()
                .lambda().eq(DiskCloudDiskUser::getCatalogueId,id)
                .eq(DiskCloudDiskUser::getOrgId,userBean.getOrgCode())
                .eq(DiskCloudDiskUser::getDeleteFlag,0)
        );
        return ResultUtil.data(userattgor);
    }
    /**
     * 云盘-退出空间
     *
     * @param userBean
     * @return
     */
    @PostMapping(value = "/sign_out/{id}")
    @ApiOperation(value = "退出共享空间", httpMethod = "POST", notes = "退出共享空间")
    public Result<Object> signOut(@CurrentUser UserBean userBean, @PathVariable Integer id) {
        try {
            DiskCloudDiskUser diskCloudDiskUser = DiskCloudDiskUser.builder().id(id).build().selectById();
            if(StringUtils.isEmpty(diskCloudDiskUser)){
                ResultUtil.error("操作失败，该共享空间不存在");
            }
            DiskCloudDiskUser entity = DiskCloudDiskUser.builder().build();
            entity.setId(diskCloudDiskUser.getId());
            entity.setDeleteFlag(1);
            boolean flag = entity.updateById();
            if(flag){
                return ResultUtil.success();
            }else {
                return ResultUtil.error("操作失败!");
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("获取我的文件夹列表异常");
        }
    }

    /**
     * 云盘-删除文件夹以及文件
     *
     * @param userBean
     * @return
     */
    @PostMapping(value = "/edit_catalogue")
    @ApiOperation(value = "删除文件夹以及文件", httpMethod = "POST", notes = "删除文件夹以及文件")
    public Result<Object> editCatalogue(@CurrentUser UserBean userBean, @RequestBody DiskCatalogueDto diskCatalogueDto) {
        try {
            DiskCatalogue diskCatalogue = new DiskCatalogue();
            DiskCatalogue catalogue = DiskCatalogue.builder().id(diskCatalogueDto.getId()).build().selectById();
            if(StringUtils.isEmpty(catalogue)){
                ResultUtil.error("操作失败~");
            }
            // 云盘逻辑删除 权限判断
            if(JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueDto.getType()) &&
                    !StringUtils.isEmpty(diskCatalogueDto.getId()) && diskCatalogueDto.getId() > 0){
                diskCatalogueDto.setUserId(userBean.getEmpNum());
                if(!diskCloudDiskUserService.getDiskUserRole(diskCatalogueDto,DiskCatalogue.builder().id(diskCatalogueDto.getId()).build().selectById())){
                    return ResultUtil.error("无权限，操作失败!");
                }
                // 删除之前的成员 全部重新添加
                DiskCloudDiskUser.builder().deleteFlag(1).build().update(new QueryWrapper<DiskCloudDiskUser>()
                        .lambda().eq(DiskCloudDiskUser::getOrgId, userBean.getOrgCode())
                        .eq(DiskCloudDiskUser::getCatalogueId, catalogue.getId()));

            }
            diskCatalogue.setId(catalogue.getId());
            diskCatalogue.setDeleteFlag(1);
            diskCatalogue.setUpdateUserId(userBean.getEmpNum());
            boolean flag = diskCatalogue.updateById();

            if(flag){
                return ResultUtil.success();
            }else {
                return ResultUtil.error("操作失败!");
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("删除文件夹以及文件异常");
        }
    }
}
