package cn.timer.api.controller.disk.sevice;

import cn.timer.api.bean.disk.DiskCatalogue;
import cn.timer.api.bean.disk.DiskCatalogueFiles;
import cn.timer.api.bean.disk.DiskFiles;
import cn.timer.api.dao.disk.DiskCatalogueMapper;
import cn.timer.api.dto.disk.DiskCatalogueParam;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wuqingjun
 * @email 284718418@qq.com
 * @date 2021/12/29
 */
@Service
public class DiskCatalogueServiceImpl implements DiskCatalogueService {
    //所有文件夹
    private static List<DiskCatalogue> treeList = new ArrayList<>();
    //所有文件
    private static List<DiskFiles> filesList = new ArrayList<>();
    //所有文件夹ID
    private static Map<Integer, List<DiskCatalogueFiles>> map = new HashMap<Integer, List<DiskCatalogueFiles>>();

    @Resource
    private DiskCatalogueMapper diskCatalogueMapper;

    @Override
    public List<DiskCatalogue> getDiskCatalogueList(Integer orgId, Integer userId, Integer type) {
        treeList = DiskCatalogue.builder().build().selectList(new QueryWrapper<DiskCatalogue>()
                .lambda().eq(DiskCatalogue::getOrgId, orgId)
                .eq(DiskCatalogue::getCreateUserId, userId)
                .eq(DiskCatalogue::getType, type)
                .eq(DiskCatalogue::getDeleteFlag, 0)
        );
        if (treeList.size() == 0) {
            return treeList;
        }
        filesList = DiskFiles.builder().build().selectList(new QueryWrapper<DiskFiles>()
                .lambda().eq(DiskFiles::getCreateUserId, userId)
                .eq(DiskFiles::getDeleteFlag, 0)

        );

        List<DiskCatalogueFiles> diskCatalogueFilesList = DiskCatalogueFiles.builder().build().selectList(new QueryWrapper<DiskCatalogueFiles>()
                .lambda().in(DiskCatalogueFiles::getCatalogueId, treeList.stream().map(item -> item.getId()).collect(Collectors.toList()))
                .eq(DiskCatalogueFiles::getDeleteFlag, 0)

        );
        //根据CatalogueId分组
        map = diskCatalogueFilesList.stream().collect(Collectors.groupingBy(DiskCatalogueFiles::getCatalogueId));

        List<DiskCatalogue> list = treeList.stream().filter(s -> s.getParentId() == 0).collect(Collectors.toList());

        return getTree(list);
    }

    @Override
    public List<DiskCatalogue> getDiskCatalogue(DiskCatalogueParam diskCatalogueParam) {
        //diskCatalogueMapper.
        return null;
    }

    /**
     * 递归获取树形结构
     *
     * @param parentList 一级节点
     * @return
     */
    public static List<DiskCatalogue> getTree(List<DiskCatalogue> parentList) {
        List<DiskCatalogue> list = new ArrayList<>();
        if (parentList != null) {
            for (int i = 0; i < parentList.size(); i++) {
                list.add(recursiveTree(parentList.get(i).getId()));
            }
        }
        return list;
    }


    /**
     * 递归算法解析成树形结构
     *
     * @param cid
     */
    public static DiskCatalogue recursiveTree(Integer cid) {
        DiskCatalogue node = getMenuById(cid);
        //填充文件夹下所有文件
        List<DiskCatalogueFiles> diskCatalogueFilesList = (List<DiskCatalogueFiles>) map.get(cid);
        if (!CollectionUtils.isEmpty(diskCatalogueFilesList)) {
            List<Integer> ids = diskCatalogueFilesList.stream().map(item -> item.getFilesId()).collect(Collectors.toList());
            List<DiskFiles> diskFilesList = filesList.stream().filter(item -> ids.contains(item.getId())).collect(Collectors.toList());
            node.setDiskFilesList(diskFilesList);
        }
        List<DiskCatalogue> childTreeNodes = getChildTreeById(cid);
        for (DiskCatalogue child : childTreeNodes) {
            DiskCatalogue n = recursiveTree(child.getId());
            List<DiskCatalogue> list = new ArrayList<DiskCatalogue>();
            list.add(n);
            if (node.getChildren() == null) {
                List<DiskCatalogue> li = new ArrayList<DiskCatalogue>();
                node.setChildren(li);
            }
            node.getChildren().add(n);
        }
        return node;
    }

    /**
     * 根据CID查询节点对象
     */
    public static DiskCatalogue getMenuById(Integer cid) {
        Map map = getTreeMap();
        return (DiskCatalogue) map.get(cid);
    }

    /**
     * 一次性取所有数据，为了减少对数据库查询操作
     *
     * @return
     */
    public static Map getTreeMap() {
        Map map = new HashMap<Integer, DiskCatalogue>();
        if (null != treeList) {
            for (DiskCatalogue d : treeList) {
                map.put(d.getId(), d);
            }
        }
        return map;
    }

    /**
     * 根据父节点CID获取所有子节点
     */
    public static List<DiskCatalogue> getChildTreeById(Integer cid) {
        List<DiskCatalogue> list = new ArrayList<>();
        if (null != treeList) {
            for (DiskCatalogue d : treeList) {
                if (null != cid) {
                    if (cid.equals(Integer.valueOf(d.getParentId()))) {
                        list.add(d);
                    }
                }
            }
        }
        return list;
    }

}
